"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.BuildCommand = void 0;
const core_1 = require("@rspack/core");
const util = __importStar(require("util"));
const fs = __importStar(require("fs"));
const options_1 = require("../utils/options");
class BuildCommand {
    async apply(cli) {
        cli.program.command(["build [entry..]", "$0", "bundle", "b"], "run the rspack build", yargs => (0, options_1.commonOptions)(yargs).options({
            analyze: {
                type: "boolean",
                default: false,
                describe: "analyze"
            },
            json: {
                describe: "emit stats json"
            }
        }), async (options) => {
            var _a;
            const config = await cli.loadConfig(options);
            if (options.analyze) {
                const { BundleAnalyzerPlugin } = await Promise.resolve().then(() => __importStar(require("webpack-bundle-analyzer")));
                ((_a = config.plugins) !== null && _a !== void 0 ? _a : (config.plugins = [])).push({
                    name: "rspack-bundle-analyzer",
                    apply(compiler) {
                        new BundleAnalyzerPlugin({
                            generateStatsFile: true,
                            // TODO: delete this once runtime refacted.
                            excludeAssets: "runtime.js"
                        }).apply(compiler);
                    }
                });
            }
            console.time("build");
            const stats = await util.promisify(core_1.rspack)(config);
            const statsJson = stats.toJson();
            if (statsJson.errors.length > 0) {
                throw new Error(statsJson.errors.map(x => x.message).join("\n"));
            }
            console.timeEnd("build");
            if (options.json) {
                const { stringifyStream: createJsonStringifyStream } = await Promise.resolve().then(() => __importStar(require("@discoveryjs/json-ext")));
                const logger = cli.getLogger();
                const handleWriteError = error => {
                    logger.error(error);
                    process.exit(2);
                };
                if (options.json === true) {
                    createJsonStringifyStream(statsJson)
                        .on("error", handleWriteError)
                        .pipe(process.stdout)
                        .on("error", handleWriteError)
                        .on("close", () => process.stdout.write("\n"));
                }
                else if (typeof options.json === "string") {
                    createJsonStringifyStream(statsJson)
                        .on("error", handleWriteError)
                        .pipe(fs.createWriteStream(options.json))
                        .on("error", handleWriteError)
                        // Use stderr to logging
                        .on("close", () => {
                        process.stderr.write(`[rspack-cli] ${cli.colors.green(`stats are successfully stored as json to ${options.json}`)}\n`);
                    });
                }
            }
        });
    }
}
exports.BuildCommand = BuildCommand;
//# sourceMappingURL=build.js.map